// CoreBitcoin by Oleg Andreev <oleganza@gmail.com>, WTFPL.

#import <Foundation/Foundation.h>
#import "BTCPaymentRequest.h"
#import "BTCPaymentMethodDetails.h"
#import "BTCPaymentMethod.h"

extern NSInteger const BTCPaymentMethodRequestVersion1;

@interface BTCPaymentMethodRequest : NSObject

// Version of the payment request and payment details.
// Default is BTCPaymentRequestVersion1.
@property(nonatomic, readonly) NSInteger version;

// Public-key infrastructure (PKI) system being used to identify the merchant.
// All implementation should support "none", "x509+sha256" and "x509+sha1".
// See BTCPaymentRequestPKIType* constants.
@property(nonatomic, readonly, nonnull) NSString* pkiType;

// PKI-system data that identifies the merchant and can be used to create a digital signature.
// In the case of X.509 certificates, pki_data contains one or more X.509 certificates.
// Depends on pkiType. Optional.
@property(nonatomic, readonly, nullable) NSData* pkiData;

// A BTCPaymentDetails object.
@property(nonatomic, readonly, nonnull) BTCPaymentMethodDetails* details;

// Digital signature over a hash of the protocol buffer serialized variation of
// the PaymentRequest message, with all serialized fields serialized in numerical order
// (all current protocol buffer implementations serialize fields in numerical order) and
// signed using the private key that corresponds to the public key in pki_data.
// Optional fields that are not set are not serialized (however, setting a field to its default value will cause it to be serialized and will affect the signature).
// Before serialization, the signature field must be set to an empty value so that
// the field is included in the signed PaymentRequest hash but contains no data.
@property(nonatomic, readonly, nullable) NSData* signature;

// Array of DER encoded certificates or nil if pkiType does offer certificates.
// This list is extracted from raw `pkiData`.
// If set, certificates are cerialized in X509Certificates object and set to pkiData.
@property(nonatomic, readonly, nonnull) NSArray* certificates;

// A date against which the payment request is being validated.
// If nil, system date at the moment of validation is used.
@property(nonatomic, nullable) NSDate* currentDate;

// Returns YES if payment request is correctly signed by a trusted certificate if needed
// and expiration date is valid.
// Accessing this property also updates `status` and `signerName`.
@property(nonatomic, readonly) BOOL isValid;

// Human-readable name of the signer or nil if it's unsigned.
// You should display this to the user as a name of the merchant.
// Accessing this property also updates `status` and `isValid`.
@property(nonatomic, readonly, nullable) NSString* signerName;

// Validation status.
// Accessing this property also updates `commonName` and `isValid`.
@property(nonatomic, readonly) BTCPaymentRequestStatus status;

// Binary serialization in protocol buffer format.
@property(nonatomic, readonly, nonnull) NSData* data;

- (nullable id) initWithData:(nullable NSData*)data;

@end

